# -*- coding: utf-8 -*-
#
# modulemd_summarizer - Prints a summary of ModuleMD files
#
# Copyright © 2018, 2019 Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
# Rafael dos Santos <rdossant@redhat.com>
# Nils Philippsen <nils@redhat.com>

import logging
import tempfile

from click import ClickException
import gi
gi.require_version('Modulemd', '1.0')  # noqa: E402
from gi.repository import Modulemd
import smartcols

from . import _fetchrepodata
from .config import runtime_config


log = logging.getLogger(__name__)


def _print_summary(module_builds, module_defaults, as_tree):
    tb = smartcols.Table()
    cl = tb.new_column('Name')
    cl.tree = as_tree
    cl_strm = tb.new_column('Stream')
    cl_ver = tb.new_column('Version')
    cl_ctxt = tb.new_column('Context')
    cl_prof = tb.new_column('Profiles')
    cl_deps = tb.new_column('Dependencies')
    parent_ln = {}

    mnames_defaults = {d.props.module_name: d for d in module_defaults}

    sorted_builds = sorted(module_builds, key=lambda mb: mb.props.name)

    for mbuild in sorted_builds:
        mbp = mbuild.props
        mdp = mnames_defaults.get(mbp.name,
                                  Modulemd.Defaults(module_name=mbp.name)).props

        if as_tree:
            pl = parent_ln.get(mbp.name, None)
            if pl is None:
                pl = parent_ln.setdefault(mbp.name, tb.new_line())
            pl[cl] = mbp.name
        else:
            pl = None

        ln = tb.new_line(pl)
        ln[cl] = mbp.name
        ln[cl_strm] = mbp.stream + (
            ' [d]' if mbp.stream == mdp.default_stream else '')
        ln[cl_ver] = str(mbp.version)
        # old module builds don't have the context set in their modulemd
        ln[cl_ctxt] = mbp.context or '00000000'

        streams_dprofiles = {k: v.get()
                             for k, v in mdp.profile_defaults.items()}

        ln[cl_prof] = ', '.join(sorted(
            p + ' [d]'
            if p in streams_dprofiles.get(mbp.stream, ())
            else p
            for p in mbp.profiles
        ))

        dlist = sorted(
            f"{dname}:{dstream}"
            for deps in mbp.dependencies
            for dname, streams_ss in sorted(deps.props.requires.items(),
                                            key=lambda x: x[0])
            for dstream in sorted(streams_ss.get())
        )

        if dlist:
            ln[cl_deps] = ", ".join(dlist)

    if sorted_builds:
        print(tb)
        print('\nHint: [d]efault')
    else:
        print("No builds found!")


def summarize_modules(mfilter=None, yamls=None, urls=None, as_tree=False):
    """
    Load Modulemd objects from each yaml file in the `yamls` list or each url
    in `urls` list and print a summary of the modules found with streams,
    context, version, profiles and dependencies information.
    If no files and no ulrs are passed, local cached metadata is used instead.

    *mfilter*: if present, restricts output to the nsvc supplied. Glob pattern
               can be used, e.g for filtering by stream: '*:master:*'

    *yamls*: list of yaml files to parse. If any, locally cached metadata will
             be ignored

    *urls*: list of repositories to read from. If any, locally cached metadata
            will be ignored.

    *as_tree*: print the summary in tree format, grouping information.
    """

    module_builds = []
    module_defaults = []

    if yamls or urls:
        if yamls:
            for yaml in yamls:
                try:
                    objects = Modulemd.objects_from_file(yaml)
                except gi.repository.GLib.GError as gerror:
                    raise ClickException(f"Could not read {yaml}: {gerror}") from gerror
        elif urls:
            for url in urls:
                with tempfile.TemporaryDirectory() as local_path:
                    rp = _fetchrepodata.RepoPaths(url, local_path)
                    _fetchrepodata._download_metadata_files(rp)
                    objects = _fetchrepodata._read_modules(rp)

        # distribute modules/builds and defaults into their own lists
        for o in objects:
            if isinstance(o, Modulemd.Module):
                module_builds.append(o)
            elif isinstance(o, Modulemd.Defaults):
                module_defaults.append(o)
            else:
                log.warning(f"Encountered unknown object in YAML: {o!r}")
    else:
        source = runtime_config['source']
        source_type = runtime_config['source_type']

        if source_type == 'buildsys':
            log.info("Querying build systems for information, this can take some time.")

        if mfilter:
            module_patterns = [p.split(":", 1)[0] for p in mfilter]
        else:
            module_patterns = None

        module_builds = list(source.enumerate_module_builds(nsvc_patterns=mfilter))
        module_defaults = list(source.enumerate_module_defaults(
            module_patterns=module_patterns
        ))

    _print_summary(module_builds, module_defaults, as_tree)
