"""Tests for the general behaviour of the CLI"""

import json
import os
import re
import subprocess
import sys

import pytest

from _fedmod._fetchrepodata import CACHEDIR


def _run_fedmod(args):
    # Run via the -m switch to ensure we get the expected version of fedmod
    cmd = [sys.executable, "-m", "_fedmod"]
    cmd.extend(args)
    return subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)


class TestMissingMetadata(object):

    def setup(self):
        self._moved_cache_dir = _moved_cache_dir = CACHEDIR + "__"
        try:
            os.rename(CACHEDIR, _moved_cache_dir)
        except FileNotFoundError:
            self.restore_cache_dir = False
        else:
            self.restore_cache_dir = True

    def teardown(self):
        if self.restore_cache_dir:
            os.rename(self._moved_cache_dir, CACHEDIR)
        self.restore_cache_dir = False

    def test_missing_metadata_handling(self, capfd):
        result = _run_fedmod(["list-modules"])
        assert result.returncode == 2
        assert result.stdout == b""
        expected_err_msg = (
            b"'<filename>' does not exist. Run `fedmod fetch-metadata`.\n")
        err_msg = re.sub(b"'.*'", b"'<filename>'", result.stderr)
        assert err_msg == expected_err_msg


EXPECTED_SUBCOMMANDS = (
    'rpm2module',
    'fetch-metadata',
    'list-modules',
    'list-rpms',
    'resolve-deps',
    'module-packages',
    'where-is-package',
    'srpm-of-rpm',
)


class TestHelpMessages(object):
    # We don't test the details of the help messages, as we trust click
    # to handle that correctly.
    #
    # Instead, these tests just ensure that all expected subcommands exist
    # and that they all accept an explicit `--help` option

    def test_default_help(self):
        assert _run_fedmod(["--help"]).returncode == 0

    def test_subcommand_help(self):
        for subcommand in EXPECTED_SUBCOMMANDS:
            assert _run_fedmod([subcommand, "--help"]).returncode == 0


class TestJSONFormatting(object):
    # Test that commands that emit JSON, emit well-formed JSON
    # TODO: extract the JSON schema definition & validation code from
    #       fabric8-analytics, and re-use it in fedmod

    @pytest.mark.needs_metadata
    def test_resolve_deps_json(self):
        result = _run_fedmod(["resolve-deps", "--json", "graphite-web"])
        assert result.returncode == 0
        assert result.stdout.endswith(b"\n")
        deps = json.loads(result.stdout)
        for dep in deps:
            assert dep.keys() == set(["rpm", "srpm", "requires"])
            for requirement, provider in dep["requires"].items():
                assert isinstance(requirement, str)
                assert isinstance(provider, list)
