"""In-process tests for the module summary functionality"""

import os.path
import re

import pytest

from _fedmod._repodata import dataset_name
from _fedmod.config import runtime_config
from _fedmod.modulemd_summarizer import summarize_modules
from _fedmod.sources import Source


testfiles_dir = os.path.join(os.path.dirname(__file__), 'files')
spec_v2_yaml_path = os.path.join(testfiles_dir, 'spec.v2.yaml')


@pytest.mark.needs_metadata
class TestModuleSummary(object):

    def setup(self):
        self.runtime_config_backup = runtime_config.copy()
        runtime_config.clear()
        runtime_config.update({
            'dataset': dataset_name,
            'source_type': 'rpmrepodata',
            'source': Source.from_type('rpmrepodata'),
        })

    def teardown(self):
        runtime_config.clear()
        runtime_config.update(self.runtime_config_backup)

    def matches(self, mod, strm, ver, ctxt, prof, deps, out):
        mstr = fr'^{mod}\s+{strm}\s+{ver}\s+{ctxt}\s+{prof}\s+{deps}$'
        return re.search(mstr, out, re.M) is not None

    # FIXME: we should mock the fetched metadata so that these tests do not
    # fail when the metadata changes
    def test_summarize_modules(self, capfd):
        summarize_modules()
        out, err = capfd.readouterr()

        assert self.matches('reviewboard', '2.5', '.*', '083bce86',
                            r'default \[d\], server',
                            'django:1.6, platform:f29', out)
        assert self.matches('reviewboard', '3.0', '.*', '083bce86',
                            r'default \[d\], server',
                            'django:1.6, platform:f29', out)
        assert self.matches('testmodule', 'master', '.*',
                            'c2c572ec', 'default', 'platform:f29', out)

    def test_summarize_modules_restricted(self, capfd):
        summarize_modules(['reviewboard*', 'django*'])
        out, err = capfd.readouterr()

        assert self.matches('reviewboard', '2.5', '.*', '083bce86',
                            r'default \[d\], server',
                            'django:1.6, platform:f29', out)
        assert self.matches('reviewboard', '3.0', '.*', '083bce86',
                            r'default \[d\], server',
                            'django:1.6, platform:f29', out)
        assert self.matches('django', '1.6', '.*', '6c81f848',
                            r'default \[d\], python2_development',
                            'platform:f29', out)
        assert not self.matches('testmodule', 'master', '.*',
                                'c2c572ec', 'default', 'platform:f29', out)

    def test_summarize_modules_local_files(self, capfd):
        summarize_modules(yamls=[spec_v2_yaml_path])
        out, err = capfd.readouterr()

        assert self.matches('foo', 'stream-name', '.*', 'c0ffee43',
                            'buildroot, container, default, minimal, ' +
                            'srpm-buildroot',
                            'compatible:v3, compatible:v4, moreextras:bar, '
                            'moreextras:foo, platform:-epel7, platform:-f27, '
                            'platform:-f28, platform:epel7, platform:f27, '
                            'platform:f28, runtime:a, runtime:b',
                            out)
